# -*- coding: utf-8 -*-
"""
API endpoints for Course Management view.
"""
from __future__ import unicode_literals

import json
import re

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseNotFound, JsonResponse
from django.views.decorators.csrf import ensure_csrf_cookie
from django.views.decorators.http import require_http_methods
from opaque_keys.edx.keys import CourseKey  # pylint: disable=import-error
from rest_framework import status

from eox_core.edxapp_wrapper.courses import (
    get_course_details_fields,
    get_courses_accessible_to_user,
    get_first_course_key,
    get_process_courses_list,
)
from eox_core.edxapp_wrapper.edxmako_module import render_to_response
from eox_core.edxapp_wrapper.users import get_course_team_user

from .helpers import enable_course_management_view


@enable_course_management_view
@login_required
@ensure_csrf_cookie
@require_http_methods(["GET"])
def management_view(request):
    """
    Renders the course management view.
    """
    if 'text/html' in request.META.get('HTTP_ACCEPT', ''):
        details_fields = get_course_details_fields()
        course_key = get_first_course_key()
        request_timeout_value = getattr(settings, 'EOX_CORE_COURSE_MANAGEMENT_REQUEST_TIMEOUT', 500)

        return render_to_response(u'management.html', {
            'course_key': course_key,
            'details_fields': details_fields,
            'request_timeout_value': request_timeout_value
        })
    return HttpResponseNotFound()


@enable_course_management_view
@login_required
@ensure_csrf_cookie
@require_http_methods(["POST", "DELETE"])
def course_team(request, *args, **kwargs):  # pylint: disable=too-many-locals
    """
    **Use Cases**
        Add or change the course team status role.

    **Example Requests**:
        POST, DELETE <studio-url>/eox-core/management/course-team-management

    **Request Values**

        * org: Organization short name.
        * user: user email.
        * role: user role (staff, instructor, '')

        Note: If the role value is empty, then it must be sent as a DELETE operation.

    **Response Values**

        * message: Status message of the operation.
        * status: Global operaion status.
            Failed, Success, No complete.
        * failed_tasks: Array containing the course failed tasks.
            message: Failed message reason.
            course: Plain course key.
        * complete_tasks: Array containing the course failed tasks.
            message: 'success'
            course: Plain course key.
    """

    json_content = get_json_from_body_request(request)
    if not json_content:
        data = {
            'message': 'Invalid request JSON',
            'status': 'Failed'
        }
        return JsonResponse(data, status=status.HTTP_400_BAD_REQUEST)

    org = json_content.get('org', '')
    user_email = json_content.get('user')
    courses_iter, in_process_course_actions = get_courses_accessible_to_user(request, org)
    active_courses, archived_courses = get_process_courses_list(courses_iter, in_process_course_actions)  # pylint: disable=unused-variable

    if not org:  # pylint: disable=no-else-return
        data = {
            'message': 'Incorrect organization name.',
            'status': 'Failed'
        }
        return JsonResponse(data, status=status.HTTP_400_BAD_REQUEST)
    elif not active_courses:
        data = {
            'message':
                'The organization {} does not exists, or does not have active courses.'.format(org),
            'status': 'Failed'
        }
        return JsonResponse(data, status=status.HTTP_404_NOT_FOUND)

    data = {
        'failed_tasks': [],
        'complete_tasks': [],
        'message': 'Operation complete.',
        'status': 'Success'
    }

    # Loop over each organization course.
    for course in active_courses:
        plain_course_key = course.get('course_key')
        course_key = CourseKey.from_string(plain_course_key)
        course_team_response = get_course_team_user(request, course_key, user_email)
        per_course_data = {}

        if course_team_response.status_code != 204:
            error_messages = json.loads(course_team_response.content)
            per_course_data = {
                'message': error_messages.get('error'),
                'course': plain_course_key
            }
            data.get('failed_tasks').append(per_course_data)
        else:
            per_course_data = {
                'message': 'success',
                'course': plain_course_key
            }
            data.get('complete_tasks').append(per_course_data)

    if data['failed_tasks']:
        data['message'] = 'Some issues were found.'
        data['status'] = 'No complete'
    return JsonResponse(data)


@enable_course_management_view
@login_required
@ensure_csrf_cookie
@require_http_methods(["GET"])
def get_courses_by_course_regex(request, *args, **kwargs):
    """
    **Use Cases**
        Get a list of course keys that match with the provided query string.
        E.g:
            course-v1:FP+DAT248x+2018*
            course-v1:FP+DAT*
            course-v1:FP*
            *DAT248x+2018*


    **Example Requests**:
        GET <studio-url>/eox-core/management/get_courses?search=<query-string>

    **Request Values**

        * search: Query string of the desired course matching.

    **Response Values**

        * message: Status message of the operation.
        * status: Global operaion status.
            Failed, Success.
        * courses: List containing the course/s key/s that match with the provided query string.
    """

    input_course_regex = request.GET.get('search')
    if not input_course_regex:
        data = {
            'message': 'No course regex provided.',
            'status': 'Failed'
        }
        return JsonResponse(data, status=status.HTTP_400_BAD_REQUEST)

    match_courses = []
    course_regex = get_course_regex(input_course_regex)

    courses_iter, in_process_course_actions = get_courses_accessible_to_user(request)
    active_courses, archived_courses = get_process_courses_list(courses_iter, in_process_course_actions)  # pylint: disable=unused-variable

    if not active_courses:
        data = {
            'message': 'No courses were found.',
            'status': 'Failed',
            'courses': []
        }
        return JsonResponse(data, status=status.HTTP_404_NOT_FOUND)

    for course in active_courses:
        plain_course_key = course.get('course_key')
        match = course_regex.search(plain_course_key)
        if match:
            match_courses.append(plain_course_key)

    if not match_courses:
        data = {
            'message': 'No courses match with this: "{}" query string.'.format(
                input_course_regex
            ),
            'status': 'Failed',
            'courses': []
        }
        return JsonResponse(data, status=status.HTTP_200_OK)

    data = {
        'message': '{} course/s were found.'.format(
            len(match_courses)
        ),
        'status': 'Success',
        'courses': match_courses
    }

    return JsonResponse(data, status=status.HTTP_200_OK)


def get_json_from_body_request(request):
    """
    Function to convert request.body into a json format.
    """
    json_content = {}
    if 'application/json' in request.META.get('CONTENT_TYPE', '') and request.body:
        try:
            json_content = json.loads(request.body)
        except ValueError:
            return json_content
    return json_content


def get_course_regex(input_course_regex):
    """
    Returns the final course regex to match with the courses keys.

    Since the input string could be a course key like value (course-v1:FP+DAT*),
    we need to replace '+' and wildcard '*' characters to '.*' character in order to build
    a correct regular expression string.
    """
    search_value_replaced = input_course_regex.replace('*', '.*')
    splitted_search_value = search_value_replaced.split('+')
    final_regex_string = '.*'.join(splitted_search_value)
    course_regex = re.compile(r"{}".format(final_regex_string))

    return course_regex



""" urls.py """

from django.conf.urls import include, url

from eox_core import views
from eox_core.api_schema import docs_ui_view

app_name = 'eox_core'  # pylint: disable=invalid-name

urlpatterns = [  # pylint: disable=invalid-name
    url(r'^eox-info$', views.info_view),
    url(r'^api/', include('eox_core.api.urls', namespace='eox-api')),
    url(r'^data-api/', include('eox_core.api.data.v1.urls', namespace='eox-data-api')),
    url(r'^management/', include('eox_core.cms.urls', namespace='eox-course-management')),
    url(r'^api-docs/$', docs_ui_view, name='apidocs-ui'),
]



"""
API v1 serializers.
"""
# pylint: disable=abstract-method
from __future__ import absolute_import, unicode_literals

from collections import OrderedDict

from django.conf import settings
from rest_framework import serializers

from eox_core.edxapp_wrapper.coursekey import get_valid_course_key, validate_org
from eox_core.edxapp_wrapper.enrollments import check_edxapp_enrollment_is_valid
from eox_core.edxapp_wrapper.users import (
    check_edxapp_account_conflicts,
    get_user_read_only_serializer,
    get_user_signup_source,
    get_username_max_length,
)

UserSignupSource = get_user_signup_source()  # pylint: disable=invalid-name

MAX_SIGNUP_SOURCES_ALLOWED = 1

USERNAME_MAX_LENGTH = get_username_max_length()


class EdxappWithWarningSerializer(serializers.Serializer):
    """
    Mixin serializer to add a warning field to Edxapp serializers
    """
    def __init__(self, *args, **kwargs):
        """
        Conditionally adds a warning field if a context is passed
        """
        super(EdxappWithWarningSerializer, self).__init__(*args, **kwargs)

        # This field should only exist if a context is actually passed to the serializer
        if self.context:
            self.fields['warning'] = serializers.SerializerMethodField(read_only=True)

    warning = serializers.CharField(read_only=True, required=False)

    def get_warning(self, obj):
        """Set the warning from the context"""
        if self.context:
            return self.context
        return None


class EdxappUserSerializer(serializers.Serializer):
    """
    Handles the serialization of the user data required to create an edxapp user
    on different backends
    """

    email = serializers.EmailField()
    username = serializers.CharField(max_length=USERNAME_MAX_LENGTH)
    password = serializers.CharField(
        style={'input_type': 'password'},
        write_only=True,
    )
    fullname = serializers.CharField(max_length=255, write_only=True)  # write_only so the user object does not complain for having the name at get_full_name()

    # Extra info to be returned by the api after creating the user
    is_active = serializers.BooleanField(read_only=True)
    is_staff = serializers.BooleanField(read_only=True)
    is_superuser = serializers.BooleanField(read_only=True)

    def validate(self, attrs):
        """
        Check that there are no conflicts on the accounts
        """
        email = attrs.get("email")
        username = attrs.get("username")
        conflicts = check_edxapp_account_conflicts(email, username)
        if conflicts:
            raise serializers.ValidationError("Account already exists with the provided: {}".format(", ".join(conflicts)))
        return attrs


class WrittableEdxappUserSerializer(EdxappUserSerializer):
    """
    Handles the serialization of the user data required to update an edxapp user.
    """
    password = serializers.CharField(
        style={'input_type': 'password'},
    )
    is_active = serializers.BooleanField()

    def validate(self, attrs):
        """
        When an update is being made, then it checks that the user:
            - Is not staff or superuser.
            - Has just one signup source.
            - The field being updated is a safe field.
        """
        # If at least one of these conditions is true, then the user can't be updated.
        safe_fields = getattr(settings, "EOX_CORE_USER_UPDATE_SAFE_FIELDS", [])

        for attr in attrs:
            if attr not in safe_fields:
                raise serializers.ValidationError({"detail": "You are not allowed to update {}.".format(attr)})

        if self.instance.is_staff or self.instance.is_superuser:
            raise serializers.ValidationError({"detail": "You can't update users with roles like staff or superuser."})

        if UserSignupSource.objects.filter(user__email=self.instance.email).count() > MAX_SIGNUP_SOURCES_ALLOWED:
            raise serializers.ValidationError({"detail": "You can't update users with more than one sign up source."})

        return attrs

    def update(self, instance, validated_data):
        """
        Update method for safe fields.
        """
        for key, value in validated_data.items():

            if key == "password":
                instance.set_password(value)
            else:
                setattr(instance, key, value)

        if validated_data:
            instance.save()

        return instance


class EdxappUserQuerySerializer(EdxappUserSerializer):
    """
    Handles the serialization of the context data required to create an edxapp user
    on different backends
    """

    activate_user = serializers.BooleanField(default=False)  # We need to allow the api to activate users later on


class EdxappEnrollmentAttributeSerializer(serializers.Serializer):
    """
    Attributes serializer
    """
    namespace = serializers.CharField()
    name = serializers.CharField()
    value = serializers.CharField()


class EdxappValidatedCourseIDField(serializers.Field):
    """
    CourseKey Field
    """
    def to_representation(self, value):
        return str(value)

    def to_internal_value(self, data):
        """
        Check course_id has the correct format and that it is allowed inside the org
        """
        if validate_org(data):  # pylint: disable=no-else-return
            return str(get_valid_course_key(data))
        else:
            raise serializers.ValidationError('Invalid course_id {}'.format(data))


class EdxappCourseEnrollmentSerializer(serializers.Serializer):
    """Serializes CourseEnrollment

    Aggregates all data from the Course Enrollment table, and pulls in the serialization for
    the Course Descriptor and course modes, to give a complete representation of course enrollment.

    """

    username = serializers.CharField(max_length=USERNAME_MAX_LENGTH, default=None, source='user')
    is_active = serializers.BooleanField(default=True)
    mode = serializers.CharField(max_length=100)
    enrollment_attributes = EdxappEnrollmentAttributeSerializer(many=True, required=False)
    course_id = EdxappValidatedCourseIDField()

    class Meta:
        """
        Add extra details for swagger
        """
        swagger_schema_fields = {
            "example": OrderedDict(
                [
                    ("username", "johndoe"),
                    ("is_active", True),
                    ("mode", "audit"),
                    ("enrollment_attributes", []),
                    ("course_id", "course-v1:edX+DemoX+Demo_Course")
                ]
            ),
        }

    def validate(self, attrs):
        """
        Check that there are no issues with enrollment
        """
        errors = check_edxapp_enrollment_is_valid(**attrs)
        if errors:
            raise serializers.ValidationError(", ".join(errors))
        return attrs


class EdxappCourseEnrollmentQuerySerializer(EdxappCourseEnrollmentSerializer):
    """
    Handles the serialization of the context data required to create an enrollment
    on different backends
    """
    username = serializers.CharField(max_length=USERNAME_MAX_LENGTH, default=None)
    email = serializers.CharField(max_length=255, default=None)
    force = serializers.BooleanField(default=False)
    course_id = EdxappValidatedCourseIDField(default=None)
    bundle_id = serializers.CharField(max_length=255, default=None)


class EdxappCoursePreEnrollmentSerializer(EdxappWithWarningSerializer):
    """Serialize CourseEnrollmentAllowed

    Handles the serialization of the context data required to create a course whitelisting or pre-enrollments for a user
    """
    course_id = EdxappValidatedCourseIDField()
    auto_enroll = serializers.BooleanField(default=True)
    email = serializers.EmailField()


def EdxappUserReadOnlySerializer(*args, **kwargs):   # pylint: disable=invalid-name
    """
    Fake class to lazily retrieve UserReadOnlySerializer
    """
    return get_user_read_only_serializer()(*args, **kwargs)


class EdxappSectionBreakdownSerializer(serializers.Serializer):
    """
    Serializes the `section_breakdown` portion of the Grades API.
    """
    attempted = serializers.BooleanField()
    assignment_type = serializers.CharField()
    percent = serializers.FloatField()
    score_earned = serializers.FloatField()
    score_possible = serializers.FloatField()
    subsection_name = serializers.CharField()


class EdxappGradingRawPolicySerializer(serializers.Serializer):
    """
    Serializes the items in the GRADER part of grading policy.
    """
    assignment_type = serializers.CharField(source='type')
    count = serializers.IntegerField(source='min_count')
    dropped = serializers.IntegerField(source='drop_count')
    weight = serializers.FloatField()


class EdxappGradingPolicySerializer(serializers.Serializer):
    """
    Serializes the course grading policy
    """
    grader = EdxappGradingRawPolicySerializer(many=True, source="GRADER")
    grade_cutoffs = serializers.DictField(child=serializers.FloatField(), source="GRADE_CUTOFFS")


class EdxappGradeSerializer(serializers.Serializer):
    """
    Serializes the grades data for a user in a given course
    """
    earned_grade = serializers.FloatField()
    grading_policy = EdxappGradingPolicySerializer(required=False)
    section_breakdown = EdxappSectionBreakdownSerializer(many=True, required=False)

    class Meta:
        """
        Add extra details for swagger
        """
        swagger_schema_fields = {
            "example":
            {
                "earned_grade": 0.5,
                "grading_policy": {
                    "grader": [
                        {
                            "assignment_type": "Homework",
                            "count": 1,
                            "dropped": 0,
                            "weight": 1.0
                        }
                    ],
                    "grade_cutoffs": {
                        "Pass": 0.5
                    }
                },
                "section_breakdown": [
                    {
                        "attempted": True,
                        "assignment_type": "Homework",
                        "percent": 0.5,
                        "score_earned": 5,
                        "score_possible": 10,
                        "subsection_name": "Homework - Questions"
                    }
                ]
            }
        }



"""
Backend for certificates app.
"""

from lms.djangoapps.certificates.models import GeneratedCertificate  # pylint: disable=import-error


def get_generated_certificate():
    """ get GeneratedCertificate model. """
    return GeneratedCertificate



#!/usr/bin/python
"""
Util function definitions.
"""
import hashlib

from django.core import cache

try:
    cache = cache.caches['general']  # pylint: disable=invalid-name
except Exception:  # pylint: disable=broad-except
    cache = cache.cache  # pylint: disable=invalid-name


def fasthash(string):
    """
    Hashes `string` into a string representation of a 128-bit digest.
    """
    md4 = hashlib.new("md4")
    md4.update(string.encode('utf-8'))
    return md4.hexdigest()



#!/usr/bin/python
"""
Test module for Utils
"""
from django.test import TestCase

from eox_core.utils import fasthash


class UtilsTest(TestCase):
    """
    Test the functions included on utils module
    """

    def test_fasthash_call(self):
        """
        Answers the question: Can we apply the fasthash method?
        """
        test_str = "test_str"
        fasthash(test_str)



""" Backend abstraction """
import os

import edxmako  # pylint: disable=import-error

from eox_core import cms


def render_to_response(template_name, dictionary=None, namespace='main', request=None, **kwargs):
    """ Custom render_to_response implementation using configurable backend and adding template dir """
    module_templates_to_include = [
        cms
    ]
    for module in module_templates_to_include:
        path_to_templates = os.path.dirname(module.__file__) + '/templates'
        if path_to_templates not in edxmako.LOOKUP['main'].directories:
            edxmako.paths.add_lookup('main', path_to_templates)
    return edxmako.shortcuts.render_to_response(template_name, dictionary, namespace, request, **kwargs)



""" Edxmako backend abstraction. """
from importlib import import_module

from django.conf import settings


def render_to_response(*args, **kwargs):
    """ Return render to response. """

    backend_function = settings.EDXMAKO_MODULE
    backend = import_module(backend_function)

    return backend.render_to_response(*args, **kwargs)



""" urls.py """

from django.conf.urls import url

from eox_core.cms.views import course_team, get_courses_by_course_regex, management_view

app_name = 'eox_core'  # pylint: disable=invalid-name

urlpatterns = [  # pylint: disable=invalid-name
    url(r'^courses$', management_view, name='management-courses-view'),
    url(r'^course-team-management$', course_team, name='course-team-management'),
    url(r'^get_courses/?$', get_courses_by_course_regex, name='get-courses-regex'),
]



"""
Helpers module.
"""

import waffle
from django.http import HttpResponseNotFound

ENABLE_COURSE_MANAGEMENT_AUTOMATION = 'course_management_automation_module'


def enable_course_management_view(function):
    """
    A decorator to check if the waffle switch is active and
    the course management view can be accessed.
    """
    def wrap(request, *args, **kwargs):
        """
        Wrap function to check if the Waffle switch is enable, if so, keep going in the normal flow,
        if not, raise a HttpResponseNotFound 404.
        """
        if not waffle.switch_is_active(ENABLE_COURSE_MANAGEMENT_AUTOMATION):
            return HttpResponseNotFound()
        return function(request, *args, **kwargs)
    return wrap



"""
Extensions to the regular defined third party auth backends
"""
import logging

from django.conf import settings

from eox_core.edxapp_wrapper.configuration_helpers import get_configuration_helper

try:
    from social_core.backends.open_id_connect import OpenIdConnectAuth
except ImportError:
    OpenIdConnectAuth = object


configuration_helper = get_configuration_helper()  # pylint: disable=invalid-name

LOG = logging.getLogger(__name__)


class ConfigurableOpenIdConnectAuth(OpenIdConnectAuth):
    """
    Generic backend that can be configured via the site settings
    """
    name = 'config-based-openidconnect'

    def __init__(self, *args, **kwargs):
        conf = configuration_helper.get_value(
            "OIDC_CONFIG",
            getattr(settings, 'OIDC_CONFIG', {})
        )

        for key in conf:
            try:
                setattr(self, key, conf.get(key, getattr(self, key)))
            except Exception:  # pylint: disable=broad-except
                LOG.error("Tried and failed to set property %s of a config-based-openidconnect", key)

        super(ConfigurableOpenIdConnectAuth, self).__init__(*args, **kwargs)



"""
API v1 views.
"""

from __future__ import absolute_import, unicode_literals

import logging

import edx_api_doc_tools as apidocs
from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from django.utils import six
from rest_framework import status
from rest_framework.authentication import SessionAuthentication
from rest_framework.exceptions import APIException, NotFound, ValidationError
from rest_framework.renderers import BrowsableAPIRenderer, JSONRenderer
from rest_framework.response import Response
from rest_framework.views import APIView

from eox_core.api.v1.permissions import EoxCoreAPIPermission
from eox_core.api.v1.serializers import (
    EdxappCourseEnrollmentQuerySerializer,
    EdxappCourseEnrollmentSerializer,
    EdxappCoursePreEnrollmentSerializer,
    EdxappGradeSerializer,
    EdxappUserQuerySerializer,
    EdxappUserReadOnlySerializer,
    EdxappUserSerializer,
    WrittableEdxappUserSerializer,
)
from eox_core.edxapp_wrapper.bearer_authentication import BearerAuthentication
from eox_core.edxapp_wrapper.coursekey import get_valid_course_key
from eox_core.edxapp_wrapper.courseware import get_courseware_courses
from eox_core.edxapp_wrapper.enrollments import create_enrollment, delete_enrollment, get_enrollment, update_enrollment
from eox_core.edxapp_wrapper.grades import get_course_grade_factory
from eox_core.edxapp_wrapper.pre_enrollments import (
    create_pre_enrollment,
    delete_pre_enrollment,
    get_pre_enrollment,
    update_pre_enrollment,
)
from eox_core.edxapp_wrapper.users import create_edxapp_user, get_edxapp_user

LOG = logging.getLogger(__name__)


class UserQueryMixin:
    """
    Provides tools to create user queries
    """

    def __init__(self, *args, **kwargs):
        """
        Defines instance attributes
        """
        super(UserQueryMixin, self).__init__(*args, **kwargs)
        self.query_params = None
        self.site = None

    def initial(self, request, *args, **kwargs):
        """
        Loads the site into the object for every kind of request.
        """
        super(UserQueryMixin, self).initial(request, *args, **kwargs)

        if hasattr(request, "site"):
            self.site = request.site

    def get_query_params(self, request):
        """
        Utility to read the query params in a forgiving way
        As a side effect it loads self.query_params also in a forgiving way
        """
        query_params = request.query_params
        if not query_params and request.data:
            query_params = request.data

        self.query_params = query_params

        return query_params

    def get_user_query(self, request, query_params=None):
        """
        Utility to prepare the user query
        """
        if not query_params:
            query_params = self.get_query_params(request)

        username = query_params.get("username", None)
        email = query_params.get("email", None)

        if not email and not username:
            raise ValidationError(detail="Email or username needed")

        user_query = {}
        if hasattr(self, "site") and self.site:
            user_query["site"] = self.site
        if username:
            user_query["username"] = username
        elif email:
            user_query["email"] = email

        return user_query


class EdxappUser(UserQueryMixin, APIView):
    """
    Handles API requests to create users
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    def post(self, request, *args, **kwargs):
        """
        Creates the users on edxapp
        """
        serializer = EdxappUserQuerySerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        data["site"] = get_current_site(request)
        user, msg = create_edxapp_user(**data)

        serialized_user = EdxappUserSerializer(user)
        response_data = serialized_user.data
        if msg:
            response_data["messages"] = msg
        return Response(response_data)

    def get(self, request, *args, **kwargs):
        """
        Retrieve an user from edxapp
        """
        query = self.get_user_query(request)
        user = get_edxapp_user(**query)
        admin_fields = getattr(settings, "ACCOUNT_VISIBILITY_CONFIGURATION", {}).get(
            "admin_fields", {}
        )
        serialized_user = EdxappUserReadOnlySerializer(
            user, custom_fields=admin_fields, context={"request": request}
        )
        response_data = serialized_user.data
        # Show a warning if the request is providing email and username
        # to let the client know we're giving priority to the username
        if "username" and "email" in self.query_params:
            response_data[
                "warning"
            ] = "The username prevails over the email when both are provided to get the user."

        return Response(response_data)


class EdxappUserUpdater(UserQueryMixin, APIView):
    """
    Handles API requests to partially update users.
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    def patch(self, request, *args, **kwargs):
        """
        Partially update a user from edxapp. Not all the fields can be updated, just the ones thought as `safe`.

        For example:

        **Requests**:
            PATCH <domain>/eox-core/api/v1/user/

        **Request body**
            {
                "email"=<USER_EMAIL>,
                "is_active": true,
                "password": "new password"
            }

        **Response values**
            User serialized.
        """
        # Pop identification
        data = request.data.copy()
        query_params = {
            "email": data.pop("email", None),
        }
        query = self.get_user_query(request, query_params=query_params)
        user = get_edxapp_user(**query)

        serializer = WrittableEdxappUserSerializer(user, data=data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        admin_fields = getattr(settings, "ACCOUNT_VISIBILITY_CONFIGURATION", {}).get(
            "admin_fields", {}
        )
        serialized_user = EdxappUserReadOnlySerializer(
            user, custom_fields=admin_fields, context={"request": request}
        )
        return Response(serialized_user.data)


class EdxappEnrollment(UserQueryMixin, APIView):
    """
    Handles API requests to create users
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    @apidocs.schema(
        body=EdxappCourseEnrollmentSerializer,
        responses={
            200: EdxappCourseEnrollmentSerializer,
            202: "User doesn't belong to site.",
            400: "Bad request, invalid course_id or missing either email or username.",
        },
    )
    def post(self, request, *args, **kwargs):
        """
        Handle creation of single or bulk enrollments

        **Example Requests**

            POST /eox-core/api/v1/enrollment/

            Request data: {
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
              "mode": "audit",
              "is_active": "False",
              "enrollment_attributes": [
                {
                  "namespace": "credit",
                  "name": "provider_id",
                  "value": "institution_name"
                }
              ]
            }

        **Parameters**

        - `username` (**required**, string, _body_):
            The username used to identify the user you want to enroll.  Use either username or email.

        - `email` (**required**, string, _body_):
            The email used to identify the user you to enroll.  Use either username or email.

        - `course_id` (**required**, string, _body_):
            The id of the course in which you want to enroll the user.

        - `mode` (**required**, string, _body_):
            The course mode for the enrollment.  Must be available for the course.

        - `is_active` (boolean, _body_):
            Flag indicating whether the enrollment is active.

        - `enrollment_attributes` (list, _body_):
            List of enrollment attributes. An enrollment attribute can be used to add extra parameters for a specific course mode.
            It must be a dictionary containing the following:
            - namespace: namespace of the attribute
            - name: name of the attribute
            - value: value of the attribute

        In case the case of bulk enrollments, you must provide a list of dictionaries containing
        the parameters specified above; the same restrictions apply.
        For example:

            [{
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
              "mode": "audit",
              "is_active": "False",
              "enrollment_attributes": [
                {
                  "namespace": "credit",
                  "name": "provider_id",
                  "value": "institution_name"
                }
              ]
             },
             {
              "email": "janedoe@example.com",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
              "mode": "audit",
              "is_active": "True",
              "enrollment_attributes": []
             },
            ]

        **Returns**

        - 200: Success, enrollment created.
        - 202: User doesn't belong to site.
        - 400: Bad request, invalid course_id or missing either email or username.
        """
        data = request.data
        return EdxappEnrollment.prepare_multiresponse(
            data, self.single_enrollment_create
        )

    @apidocs.schema(
        body=EdxappCourseEnrollmentSerializer,
        responses={
            200: EdxappCourseEnrollmentSerializer,
            202: "User or enrollment doesn't belong to site.",
            400: "Bad request, invalid course_id or missing either email or username.",
        },
    )
    def put(self, request, *args, **kwargs):
        """
        Update enrollments on edxapp

        **Example Requests**

            PUT /eox-core/api/v1/enrollment/

            Request data: {
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
              "mode": "audit",
              "is_active": "False",
              "enrollment_attributes": [
                {
                  "namespace": "credit",
                  "name": "provider_id",
                  "value": "institution_name"
                }
              ]
            }

        **Parameters**

        - `username` (**required**, string, _body_):
            The username used to identify a user enrolled on the course. Use either username or email.

        - `email` (**required**, string, _body_):
            The email used to identify a user enrolled on the course. Use either username or email.

        - `course_id` (**required**, string, _body_):
            The course id for the enrollment you want to update.

        - `mode` (**required**, string, _body_):
            The course mode for the enrollment. Must be available for the course.

        - `is_active` (boolean, _body_):
            Flag indicating whether the enrollment is active.

        - `enrollment_attributes` (list, _body_):
            An enrollment attribute can be used to add extra parameters for a specific course mode.
            It must be a dictionary containing the following:
            - namespace: namespace of the attribute
            - name: name of the attribute
            - value: value of the attribute

        **Returns**

        - 200: Success, enrollment updated.
        - 202: User or enrollment doesn't belong to site.
        - 400: Bad request, invalid course_id or missing either email or username.
        """
        data = request.data
        return EdxappEnrollment.prepare_multiresponse(
            data, self.single_enrollment_update
        )

    @apidocs.schema(
        parameters=[
            apidocs.query_parameter(
                name="username",
                param_type=str,
                description="**required**, The username used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="email",
                param_type=str,
                description="**required**, The email used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="course_id",
                param_type=str,
                description="**required**, The course id for the enrollment you want to check.",
            ),
        ],
        responses={
            200: EdxappCourseEnrollmentSerializer,
            400: "Bad request, missing course_id or either email or username",
            404: "User or course not found",
        },
    )
    def get(self, request, *args, **kwargs):
        """
        Retrieves enrollment information given a user and a course_id

        **Example Requests**

            GET /eox-core/api/v1/enrollment/?username=johndoe&
            course_id=course-v1:edX+DemoX+Demo_Course

            Request data: {
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
            }

        **Returns**

        - 200: Success, enrollment found.
        - 400: Bad request, missing course_id or either email or username
        - 404: User or course not found
        """
        user_query = self.get_user_query(request)
        user = get_edxapp_user(**user_query)

        course_id = self.query_params.get("course_id", None)

        if not course_id:
            raise ValidationError(detail="You have to provide a course_id")

        enrollment_query = {
            "username": user.username,
            "course_id": course_id,
        }
        enrollment, errors = get_enrollment(**enrollment_query)

        if errors:
            raise NotFound(detail=errors)
        response = EdxappCourseEnrollmentSerializer(enrollment).data
        return Response(response)

    @apidocs.schema(
        parameters=[
            apidocs.query_parameter(
                name="username",
                param_type=str,
                description="**required**, The username used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="email",
                param_type=str,
                description="**required**, The email used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="course_id",
                param_type=str,
                description="**required**, The course id for the enrollment you want to check.",
            ),
        ],
        responses={
            204: "Empty response",
            400: "Bad request, missing course_id or either email or username",
            404: "User or course not found",
        },
    )
    def delete(self, request, *args, **kwargs):
        """
        Delete enrollment on edxapp

        **Example Requests**

            DELETE /eox-core/api/v1/enrollment/

            Request data: {
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
            }
        """
        user_query = self.get_user_query(request)
        user = get_edxapp_user(**user_query)

        course_id = self.query_params.get("course_id", None)

        if not course_id:
            raise ValidationError(detail="You have to provide a course_id")

        delete_enrollment(user=user, course_id=course_id)
        return Response(status=status.HTTP_204_NO_CONTENT)

    def single_enrollment_create(self, *args, **kwargs):
        """
        Handle one create at the time
        """
        user_query = self.get_user_query(None, query_params=kwargs)
        user = get_edxapp_user(**user_query)

        enrollments, msgs = create_enrollment(user, **kwargs)
        # This logic block is needed to convert a single bundle_id enrollment in a list
        # of course_id enrollments which are appended to the response individually
        if not isinstance(enrollments, list):
            enrollments = [enrollments]
            msgs = [msgs]
        response_data = []
        for enrollment, msg in zip(enrollments, msgs):
            data = EdxappCourseEnrollmentSerializer(enrollment).data
            if msg:
                data["messages"] = msg
            response_data.append(data)

        return response_data

    def single_enrollment_update(self, *args, **kwargs):
        """
        Handle one update at the time
        """
        user_query = self.get_user_query(None, query_params=kwargs)
        user = get_edxapp_user(**user_query)

        course_id = kwargs.pop("course_id", None)
        if not course_id:
            raise ValidationError(detail="You have to provide a course_id for updates")
        mode = kwargs.pop("mode", None)

        return update_enrollment(user, course_id, mode, **kwargs)

    @staticmethod
    def prepare_multiresponse(request_data, action_method):
        """
        Prepare a multiple part response according to the request_data and the action_method provided
        Args:
            request_data: Data dictionary containing the query or queries to be processed
            action_method: Function to be applied to the queries (create, update)

        Returns: List of responses
        """
        multiple_responses = []
        errors_in_bulk_response = False
        many = isinstance(request_data, list)
        serializer = EdxappCourseEnrollmentQuerySerializer(data=request_data, many=many)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        if not isinstance(data, list):
            data = [data]

        for enrollment_query in data:

            try:
                result = action_method(**enrollment_query)
                # The result can be a list if the enrollment was in a bundle
                if isinstance(result, list):
                    multiple_responses += result
                else:
                    multiple_responses.append(result)
            except APIException as error:
                errors_in_bulk_response = True
                enrollment_query["error"] = {
                    "detail": error.detail,
                }
                multiple_responses.append(enrollment_query)

        if many or "bundle_id" in request_data:
            response = multiple_responses
        else:
            response = multiple_responses[0]

        response_status = status.HTTP_200_OK
        if errors_in_bulk_response:
            response_status = status.HTTP_202_ACCEPTED
        return Response(response, status=response_status)

    def handle_exception(self, exc):
        """
        Handle exception: log it
        """
        if isinstance(exc, APIException):
            LOG.error("API Error: %s", repr(exc.detail))

        return super(EdxappEnrollment, self).handle_exception(exc)


class EdxappPreEnrollment(APIView):
    """
    Handles API requests to manage whitelistings (pre-enrollments)
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    def post(self, request, *args, **kwargs):
        """
        Create whitelistings on edxapp
        """
        serializer = EdxappCoursePreEnrollmentSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        course_id = data.pop("course_id", None)
        pre_enrollment, warning = create_pre_enrollment(course_id=course_id, **data)
        response_data = EdxappCoursePreEnrollmentSerializer(
            pre_enrollment, context=warning
        ).data
        return Response(response_data)

    def put(self, request, *args, **kwargs):
        """
        Update whitelistings on edxapp
        """
        serializer = EdxappCoursePreEnrollmentSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        email = data.get("email")
        course_id = data.get("course_id")
        auto_enroll = data.pop("auto_enroll", False)

        pre_enrollment_query = {
            "email": email,
            "course_id": course_id,
        }

        pre_enrollment = get_pre_enrollment(**pre_enrollment_query)
        update_query = {
            "pre_enrollment": pre_enrollment,
            "auto_enroll": auto_enroll,
        }
        updated_pre_enrollment = update_pre_enrollment(**update_query)
        response = EdxappCoursePreEnrollmentSerializer(updated_pre_enrollment).data
        return Response(response)

    def delete(self, request, *args, **kwargs):
        """
        Delete whitelistings on edxapp
        """
        query_params = request.query_params
        if not query_params:
            query_params = request.data

        serializer = EdxappCoursePreEnrollmentSerializer(data=query_params)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        email = data.get("email")
        course_id = data.get("course_id")

        pre_enrollment_query = {
            "email": email,
            "course_id": course_id,
        }

        pre_enrollment = get_pre_enrollment(**pre_enrollment_query)
        delete_query = {
            "pre_enrollment": pre_enrollment,
        }
        delete_pre_enrollment(**delete_query)
        return Response(status=status.HTTP_204_NO_CONTENT)

    def get(self, request, *args, **kwargs):
        """
        Get whitelistings on edxapp
        """
        query_params = request.query_params
        if not query_params:
            query_params = request.data

        serializer = EdxappCoursePreEnrollmentSerializer(data=query_params)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        pre_enrollment = get_pre_enrollment(**data)
        response = EdxappCoursePreEnrollmentSerializer(pre_enrollment).data
        return Response(response)

    def handle_exception(self, exc):
        """
        Handle exception: log it
        """
        data = self.request.data or self.request.query_params
        self.log("API Error", data, exc)
        return super(EdxappPreEnrollment, self).handle_exception(exc)

    def log(self, desc, data, exception=None):
        """
        log util for this view
        """
        log_data = []
        log_data.append(desc)
        log_data.append("Exception:")
        if isinstance(exception, APIException):
            log_data.append(repr(exception.detail))
        else:
            log_data.append(repr(exception))

        log_data.append("Request data:")
        if not data:
            log_data.append("Empty request")
        else:
            for key, value in data.items():
                log_data.append("{}: {}".format(key, value))

        LOG.error(" ".join(log_data))


class EdxappGrade(UserQueryMixin, APIView):
    """
    Handles API requests to manage course grades
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    @apidocs.schema(
        parameters=[
            apidocs.query_parameter(
                name="username",
                param_type=str,
                description="**required**, The username used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="email",
                param_type=str,
                description="**required**, The email used to identify a user enrolled on the course. Use either username or email.",
            ),
            apidocs.query_parameter(
                name="course_id",
                param_type=str,
                description="**required**, The course id for the enrollment you want to check.",
            ),
            apidocs.query_parameter(
                name="detailed",
                param_type=bool,
                description="**optional**, If true include detailed data for each graded subsection",
            ),
            apidocs.query_parameter(
                name="grading_policy",
                param_type=bool,
                description="**optional**, If true include course grading policy.",
            ),
        ],
        responses={
            200: EdxappGradeSerializer,
            400: "Bad request, missing course_id or either email or username",
            404: "User, course or enrollment not found",
        },
    )
    def get(self, request):
        """
        Retrieves Grades information for given a user and course_id

        **Example Requests**

            GET /eox-core/api/v1/grade/?username=johndoe&course_id=course-v1:edX+DemoX+Demo_Course

            Request data: {
              "username": "johndoe",
              "course_id": "course-v1:edX+DemoX+Demo_Course",
            }

        **Response details**

        - `earned_grade`: Final user score for the course.
        - `section_breakdown` (**optional**): Details of each grade subsection.
            - `attempted`: Whether the learner attempted the assignment.
            - `assignment_type`: General category of the assignment.
            - `percent`: Grade obtained by the user on this subsection.
            - `score_earned`: The score a user earned on this subsection.
            - `score_possible`: Highest possible score a user can earn on this subsection.
            - `subsection_name`: Name of the subsection.
        - `grading_policy` (**optional**): Course grading policy.
            - `grade_cutoff`: Score needed to reach each grade.
            - `grader`: Details of each assignment type used by the Grader.
                - `assignment_type`: General category of the assignment.
                - `count`: Number of assignments of this type.
                - `dropped`: The number of assignments of this type that the grader will drop. The grader will drop the lowest-scored assignments first.
                - `weight`: Weight of this type of assignment towards the final grade.

        More information about grading can be found in the
        [edX documentation](https://edx.readthedocs.io/projects/open-edx-building-and-running-a-course/en/latest/student_progress/course_grades.html).

        **Returns**

        - 200: Success.
        - 400: Bad request, missing course_id or either email or username.
        - 404: User, course or enrollment not found.
        """
        user_query = self.get_user_query(request)
        user = get_edxapp_user(**user_query)

        course_id = self.query_params.get("course_id", None)
        detailed = self.query_params.get("detailed", False)
        grading_policy = self.query_params.get("grading_policy", False)

        if not course_id:
            raise ValidationError(detail="You have to provide a course_id")

        _, errors = get_enrollment(username=user.username, course_id=course_id)

        if errors:
            raise NotFound(errors)

        grade_factory = get_course_grade_factory()
        course_key = get_valid_course_key(course_id)
        course = get_courseware_courses().get_course_by_id(course_key)
        course_grade = grade_factory().read(user, course)
        response = {"earned_grade": course_grade.percent}

        if detailed in ("True", "true", "on", "1"):
            breakdown = self._section_breakdown(course_grade.subsection_grades)
            response["section_breakdown"] = breakdown
        if grading_policy in ("True", "true", "on", "1"):
            response["grading_policy"] = course.grading_policy

        return Response(EdxappGradeSerializer(response).data)

    def _section_breakdown(self, subsection_grades):
        """
        Given a list of subsections grades determine if the subsection is
        graded and creates a list of grade data for each subsection.

        This logic comes from `edunext-platform/lms/djangoapps/grades/rest_api/v1/gradebook_views`
        with a few changes in the return value: `label` and `module_id` were
        removed and `category` was renamed to `assignment_type` to keep
        consistency with the names used in `grading_policy`.
        """
        breakdown = []
        for subsection in subsection_grades.values():
            if not subsection.graded:
                continue

            attempted = False
            score_earned = 0
            score_possible = 0

            if subsection.attempted_graded or subsection.override:
                attempted = True
                score_earned = subsection.graded_total.earned
                score_possible = subsection.graded_total.possible

            breakdown.append(
                {
                    "attempted": attempted,
                    "assignment_type": subsection.format,
                    "percent": subsection.percent_graded,
                    "score_earned": score_earned,
                    "score_possible": score_possible,
                    "subsection_name": subsection.display_name,
                }
            )
        return breakdown


class UserInfo(APIView):
    """
    Auth-only view to check some basic info about the current user
    Can use Oauth2/Session
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication)
    permission_classes = (EoxCoreAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    def get(self, request, format=None):  # pylint: disable=redefined-builtin
        """
        handle GET request
        """
        content = {
            # `django.contrib.auth.User` instance.
            "user": six.text_type(request.user.username),
            "email": six.text_type(request.user.email),
            "auth": six.text_type(request.auth),
        }

        if request.user.is_staff:
            content["is_superuser"] = request.user.is_superuser
            content["is_staff"] = request.user.is_staff

        return Response(content)



"""
Certificates definitions.
"""

from importlib import import_module

from django.conf import settings


def get_generated_certificate():
    """ Gets GeneratedCertificate model. """

    backend_function = settings.EOX_CORE_CERTIFICATES_BACKEND
    backend = import_module(backend_function)

    return backend.get_generated_certificate()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Backend for the create_edxapp_user that works under the open-release/hawthorn.beta1 tag
"""
from __future__ import absolute_import, unicode_literals

import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import transaction
from openedx.core.djangoapps.lang_pref import LANGUAGE_KEY  # pylint: disable=import-error
from openedx.core.djangoapps.site_configuration import helpers as configuration_helpers  # pylint: disable=import-error
from openedx.core.djangoapps.user_api.accounts import USERNAME_MAX_LENGTH  # pylint: disable=import-error,unused-import
from openedx.core.djangoapps.user_api.accounts.api import check_account_exists  # pylint: disable=import-error
from openedx.core.djangoapps.user_api.accounts.serializers import UserReadOnlySerializer  # pylint: disable=import-error
from openedx.core.djangoapps.user_api.preferences import api as preferences_api  # pylint: disable=import-error
from rest_framework.exceptions import NotFound
from student.forms import AccountCreationForm  # pylint: disable=import-error
from student.helpers import create_or_set_user_attribute_created_on_site  # pylint: disable=import-error
from student.models import (  # pylint: disable=import-error
    UserAttribute,
    UserProfile,
    UserSignupSource,
    create_comments_service_user,
)

from student.helpers import do_create_account  # pylint: disable=import-error; pylint: disable=import-error
from student.models import CourseEnrollment  # pylint: disable=import-error; pylint: disable=import-error

LOG = logging.getLogger(__name__)
User = get_user_model()  # pylint: disable=invalid-name


def get_user_read_only_serializer():
    """
    Great serializer that fits our needs
    """
    return UserReadOnlySerializer


def check_edxapp_account_conflicts(email, username):
    """
    Exposed function to check conflicts
    """
    return check_account_exists(email=email, username=username)


def create_edxapp_user(*args, **kwargs):
    """
    Creates a user on the open edx django site using calls to
    functions defined in the edx-platform codebase

    Example call:

    data = {
        'email': "address@example.org",
        'username': "Username",
        'password': "P4ssW0rd",
        'fullname': "Full Name",
        'activate': True,
        'site': request.site,
        'language_preference': 'es-419',
    }
    user = create_edxapp_user(**data)

    """
    errors = []

    email = kwargs.pop("email")
    username = kwargs.pop("username")
    conflicts = check_edxapp_account_conflicts(email=email, username=username)
    if conflicts:
        return None, ["Fatal: account collition with the provided: {}".format(", ".join(conflicts))]

    data = {
        'username': username,
        'email': email,
        'password': kwargs.pop("password"),
        'name': kwargs.pop("fullname"),
    }
    # Go ahead and create the new user
    with transaction.atomic():
        # In theory is possible to extend the registration form with a custom app
        # An example form app for this can be found at http://github.com/open-craft/custom-form-app
        # form = get_registration_extension_form(data=params)
        # if not form:
        form = AccountCreationForm(
            data=data,
            tos_required=False,
            # TODO: we need to support the extra profile fields as defined in the django.settings
            # extra_fields=extra_fields,
            # extended_profile_fields=extended_profile_fields,
            # enforce_password_policy=enforce_password_policy,
        )
        (user, profile, registration) = do_create_account(form)  # pylint: disable=unused-variable

    site = kwargs.pop("site", False)
    if site:
        create_or_set_user_attribute_created_on_site(user, site)
    else:
        errors.append("The user was not assigned to any site")

    try:
        create_comments_service_user(user)
    except Exception:  # pylint: disable=broad-except
        errors.append("No comments_service_user was created")

    # TODO: link account with third party auth

    lang_pref = kwargs.pop("language_preference", False)
    if lang_pref:
        try:
            preferences_api.set_user_preference(user, LANGUAGE_KEY, lang_pref)
        except Exception:  # pylint: disable=broad-except
            errors.append("Could not set lang preference '{} for user '{}'".format(
                lang_pref,
                user.username,
            ))

    if kwargs.pop("activate_user", False):
        user.is_active = True
        user.save()

    # TODO: run conditional email sequence

    return user, errors


def get_edxapp_user(**kwargs):
    """
    Retrieve an user by username and/or email

    The user will be returned only if it belongs to the calling site

    Examples:
        >>> get_edxapp_user(
            {
                "username": "Bob",
                "site": request.site
            }
        )
        >>> get_edxapp_user(
            {
                "email": "Bob@mailserver.com",
                "site": request.site
            }
        )
    """
    params = {key: kwargs.get(key) for key in ['username', 'email'] if key in kwargs}
    site = kwargs.get('site')
    try:
        domain = site.domain
    except AttributeError:
        domain = None

    try:
        user = User.objects.get(**params)
        for source_method in FetchUserSiteSources.get_enabled_source_methods():
            if source_method(user, domain):
                break
        else:
            raise User.DoesNotExist
    except User.DoesNotExist:
        raise NotFound('No user found by {query} on site {site}.'.format(query=str(params), site=domain))
    return user


def get_course_team_user(*args, **kwargs):
    """
    Get _course_team_user function.
    We need to check if the SERVICE_VARIANT is equal to cms, since
    contentstore is a module registered in the INSTALLED_APPS
    of the cms only.
    """
    if settings.SERVICE_VARIANT == 'cms':
        from contentstore.views.user import _course_team_user  # pylint: disable=import-error,import-outside-toplevel
        return _course_team_user(*args, **kwargs)
    return None


class FetchUserSiteSources:
    """
    Methods to make the comparison to check if an user belongs to a site plus the
    get_enabled_source_methods that just brings an array of functions enabled to do so
    """

    @classmethod
    def get_enabled_source_methods(cls):
        """ Brings the array of methods to check if an user belongs to a site. """
        sources = configuration_helpers.get_value(
            'EOX_CORE_USER_ORIGIN_SITE_SOURCES',
            getattr(settings, 'EOX_CORE_USER_ORIGIN_SITE_SOURCES')
        )
        return [getattr(cls, source) for source in sources]

    @staticmethod
    def fetch_from_created_on_site_prop(user, domain):
        """ Fetch option. """
        if not domain:
            return False
        return UserAttribute.get_user_attribute(user, 'created_on_site') == domain

    @staticmethod
    def fetch_from_user_signup_source(user, domain):
        """ Read the signup source. """
        return len(UserSignupSource.objects.filter(user=user, site=domain)) > 0

    @staticmethod
    def fetch_from_unfiltered_table(user, site):
        """ Fetch option that does not take into account the multi-tentancy model of the installation. """
        return bool(user)


def get_course_enrollment():
    """ get CourseEnrollment model """
    return CourseEnrollment


def get_user_signup_source():
    """ get UserSignupSource model """
    return UserSignupSource


def get_user_profile():
    """ Gets the UserProfile model """

    return UserProfile



""" urls.py """

from django.conf.urls import include, url

app_name = 'eox_core'  # pylint: disable=invalid-name

urlpatterns = [  # pylint: disable=invalid-name
    url(r'^v1/', include('eox_core.api.v1.urls', namespace='eox-api')),
]



"""
Test module for users viewset.
"""
from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse
from mock import patch
from rest_framework import status
from rest_framework.test import APIClient


class UsersUpdaterAPITest(TestCase):
    """Test class for users viewset."""

    patch_permissions = patch('eox_core.api.v1.permissions.EoxCoreAPIPermission.has_permission', return_value=True)

    def setUp(self):
        """Setup method for test class."""
        self.user = User(username="test", email="test@example.com", password="testtest")
        self.client = APIClient()
        self.url = reverse("eox-api:eox-api:edxapp-user-updater")
        self.client.force_authenticate(user=self.user)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.EdxappUserReadOnlySerializer')
    def test_update_success(self, user_serializer, get_edxapp_user, signup_source, _):
        """Used to test updating safe fields of an edxapp user."""
        update_data = {
            "email": "test",
            "fullname": "test",
        }
        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertEqual(status.HTTP_200_OK, response.status_code)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    def test_update_bad_sign_up_source(self, get_edxapp_user, signup_source, _):
        """Used to test that when an user has more than one signup source then the update can't be done."""
        update_data = {
            "email": "test",
            "fullname": "test",
        }
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 2

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"detail":["You can\'t update users with more than one sign up source."]}'
                         .encode())

    @patch_permissions
    @patch('eox_core.api.v1.serializers.UserSignupSource')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    def test_update_role_not_allowed(self, get_edxapp_user, signup_source, _):
        """Used to test that if an user is staff or superuser then the update can't be done.ss"""
        user = User(username="test", email="test@example.com", password="testtest", is_staff=True)
        update_data = {
            "email": "test",
            "fullname": "test",
        }
        get_edxapp_user.return_value = user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"detail":["You can\'t update users with roles like staff or superuser."]}'
                         .encode())
